@@ -22,8 +22,8 @@ def write_config(_config_path):
     _config = {
         'floatx': _session.floatx,
         'epsilon': _session.epsilon,
-        'backend': _BACKEND,
-        'image_backend':_IMAGE_BACKEND
+        'backend': _session.backend ,
+        'image_backend':_session.image_backend
     }
     try:
         with open(_config_path, 'w') as f:
@@ -45,8 +45,8 @@ if os.path.exists(_config_path):
     assert _floatx in {'float16', 'float32', 'float64'}
     _epsilon = _config.get('epsilon', _session.epsilon)
     assert isinstance(_epsilon, float)
-    _BACKEND = _config.get('backend', _BACKEND)
-    _IMAGE_BACKEND = _config.get('image_backend', _IMAGE_BACKEND)
+    _BACKEND = _config.get('backend', _session.backend )
+    _IMAGE_BACKEND = _config.get('image_backend', _session.image_backend)
 
 
 
@@ -66,20 +66,19 @@ if not os.path.exists(_trident_dir):
 
 
 def get_backend():
-    return _BACKEND
+    return _session.backend
 
 def get_image_backend():
-    return _IMAGE_BACKEND
+    return _session.image_backend
 
 
 
 
 # Set backend based on TRIDENT_BACKEND flag, if applicable.
 if 'TRIDENT_BACKEND' in os.environ:
-    _backend = os.environ['TRIDENT_BACKEND']
-    if _backend!=_BACKEND:
-        _BACKEND = _backend
-        _session.backend = _backend
+    _BACKEND = os.environ['TRIDENT_BACKEND']
+    if _session.backend!=_BACKEND:
+        _session.backend = _BACKEND
         write_config(_config_path)
 
 
@@ -87,35 +86,26 @@ if 'TRIDENT_BACKEND' in os.environ:
 if _BACKEND == 'cntk':
     stderr.write('Using CNTK backend\n')
     stderr.write('Image Data Format: channels_first.\n')
-    stderr.write('Image Channel Order: bgr.\n')
-    _session.backend = 'pytorch'
+    stderr.write('Image Channel Order: rgb.\n')
+    _session.backend = 'cntk'
     _session.image_data_format = 'channels_first'
-    _session.image_channel_order = 'bgr'
-    from .cntk_backend import *
+    _session.image_channel_order = 'rgb'
+
+    from ..layers.cntk_normalizations import *  # from ..optims.cntk_lr_schedulers import *
+
+    #from ..optims.cntk_lr_schedulers import *
+
 
 elif _BACKEND == 'pytorch':
     stderr.write('Using Pytorch backend.\n')
     stderr.write('Image Data Format: channels_first.\n')
-    stderr.write('Image Channel Order: bgr.\n')
+    stderr.write('Image Channel Order: rgb.\n')
     _session.backend='pytorch'
     _session.image_data_format='channels_first'
-    _session.image_channel_order='bgr'
-    from .pytorch_backend import *
-    from .pytorch_ops import *
-    from ..layers.pytorch_layers import *
-    from ..layers.pytorch_activations import *
-    from ..layers.pytorch_normalizations import *
+    _session.image_channel_order='rgb'
+    #module = importlib.import_module(mName)
+    #layers=importlib.import_module('layers.pytorch_layers')
 
-    from ..layers.pytorch_constraints import *
-    from ..layers.pytorch_blocks import *
-    from ..layers.pytorch_constraints import *
-    from ..optims.pytorch_optimizers import *
-    from ..optims.pytorch_regularizers import *
-    from ..optims.pytorch_lr_schedulers import *
-    from ..optims.pytorch_losses import *
-    from ..optims.pytorch_metrics import *
-    from ..optims.pytorch_trainer import *
-    from ..data.pytorch_datasets import *
 
 elif _BACKEND == 'tensorflow':
     stderr.write('Using TensorFlow backend.\n')
@@ -124,12 +114,6 @@ elif _BACKEND == 'tensorflow':
     _session.backend = 'tensorflow'
     _session.image_data_format = 'channels_last'
     _session.image_channel_order = 'rgb'
-    from .tensorflow_backend import *
-    from ..optims.tensorflow_losses import *
-    from ..optims.tensorflow_metrics import *
-    from ..optims.tensorflow_optimizers import *
-
-
 if 'TRIDENT_IMG_BACKEND' in os.environ:
     _image_backend = os.environ['TRIDENT_IMG_BACKEND']
     if _image_backend:
@@ -151,8 +135,7 @@ elif _IMAGE_BACKEND == 'pillow':
 if not os.path.exists(_config_path):
     write_config(_config_path)
 
-
-from ..misc import *
-from .image_common import *
+from ..data.image_common import *
 from ..data.data_loaders import *
-from ..data.datasets_common import *
+from ..callbacks import *
+from ..misc.visualization_utils import *
\ No newline at end of file
