@@ -10,7 +10,7 @@ import cv2
 import numpy as np
 
 from .common import get_session,floatx
-from .load_backend import to_numpy
+
 
 
 __all__ = ['read_image','read_mask','save_image','save_mask','image2array','array2image','mask2array','array2mask','adjust_brightness','adjust_blur','adjust_saturation']
@@ -77,7 +77,7 @@ def image2array(img):
 
     '''
     if isinstance(img,str):
-        if os.path.exists(img) and img.split('.')[-1] in ('jpg','jepg','png','bmp','tiff'):
+        if os.path.exists(img) and img.split('.')[-1] in ('jpg','jpeg','png','bmp','tiff'):
             img=plt.imread(img)[::-1]
         else:
             return None
@@ -87,16 +87,18 @@ def image2array(img):
             raise ValueError('image should be 2 or 3 dimensional. Got {} dimensions.'.format(arr.ndim))
         if arr.ndim == 3:
             if arr.shape[2] in [3, 4] and arr.shape[0] not in [3, 4]:
-                pass
+
+                arr=cv2.cvtColor(arr,cv2.COLOR_BGR2RGB)
             elif arr.shape[0] in [3, 4]:
                 arr = arr.transpose([1, 2, 0])
+                arr = cv2.cvtColor(arr, cv2.COLOR_BGR2RGB)
             else:
                 raise ValueError('3d image should be 3 or 4 channel. Got {} channel.'.format(arr.shape[0]))
-        arr=img.astype(_session.floatx)[::-1]
+        #arr=img.astype(_session.floatx)[::-1]
     if arr.flags['C_CONTIGUOUS'] == False:
         arr = np.ascontiguousarray(arr)
     return arr
-def array2image(arr):
+def array2image(arr:np.ndarray):
     '''
 
     Parameters
@@ -108,7 +110,6 @@ def array2image(arr):
 
     '''
     # confirm back to numpy
-    arr = to_numpy(arr)
     if arr.ndim not in [2, 3]:
         raise ValueError('image should be 2 or 3 dimensional. Got {} dimensions.'.format(arr.ndim))
     mode = None
@@ -120,6 +121,8 @@ def array2image(arr):
             pass
         elif (_backend != 'tensorflow' and arr.shape[0] in [3, 4]):
             arr = arr.transpose([1, 2, 0])
+        elif _backend in ['pytorch', 'cntk'] and arr.ndim == 3 and arr.shape[0] in [ 3, 4]:
+            arr = arr.transpose([1, 2, 0])
         else:
             raise ValueError('3d image should be 3 or 4 channel. Got {} channel.'.format(arr.shape[0]))
 
@@ -157,7 +160,7 @@ def mask2array(img):
         arr = np.ascontiguousarray(arr)
     return arr
 
-def array2mask(arr):
+def array2mask(arr:np.ndarray):
     '''
 
     Parameters
@@ -179,7 +182,6 @@ def array2mask(arr):
     -------
 
     '''
-    arr = to_numpy(arr)
     if arr.ndim not in [2, 3]:
         raise ValueError('image should be 2 or 3 dimensional. Got {} dimensions.'.format(arr.ndim))
     arr=np.squeeze(arr)
