@@ -17,9 +17,8 @@ except ImportError:
     ImageFilter=None
 import numpy as np
 from .common import get_session
-from .load_backend import to_numpy
-
-__all__ = ['read_image','read_mask','save_image','save_mask','image2array','array2image','mask2array','array2mask','adjust_brightness','adjust_blur','adjust_saturation']
+#from .load_backend import to_numpy
+#__all__ = ['read_image','read_mask','save_image','save_mask','image2array','array2image','mask2array','array2mask','adjust_brightness','adjust_blur','adjust_saturation']
 
 
 _session=get_session()
@@ -81,7 +80,7 @@ def image2array(img):
 
     '''
     if isinstance(img,str):
-        if os.path.exists(img) and img.split('.')[-1] in ('jpg','jepg','png','bmp','tiff'):
+        if os.path.exists(img) and img.split('.')[-1] in ('jpg','jpeg','png','bmp','tiff'):
             img=pil_image.open(img)
         else:
             return None
@@ -89,6 +88,7 @@ def image2array(img):
     if isinstance(img,pil_image.Image):
         arr = np.array(img).astype(_session.floatx)
     elif isinstance(img, np.ndarray):
+        arr=img
         if arr.ndim not in [2, 3]:
             raise ValueError('image should be 2 or 3 dimensional. Got {} dimensions.'.format(arr.ndim))
         if arr.ndim == 3:
@@ -102,7 +102,9 @@ def image2array(img):
     if arr.flags['C_CONTIGUOUS'] == False:
         arr = np.ascontiguousarray(arr)
     return arr
-def array2image(arr):
+
+
+def array2image(arr:np.ndarray):
     '''
 
     Parameters
@@ -114,7 +116,7 @@ def array2image(arr):
 
     '''
     # confirm back to numpy
-    arr = to_numpy(arr)
+
     if arr.ndim not in [2, 3]:
         raise ValueError('image should be 2 or 3 dimensional. Got {} dimensions.'.format(arr.ndim))
     mode = None
@@ -123,7 +125,9 @@ def array2image(arr):
     elif arr.ndim == 3:
         if (_backend=='tensorflow' and  arr.shape[2] in [3, 4]) or (arr.shape[2] in [3, 4] and arr.shape[0] not in [3, 4]):
             pass
-        elif (_backend!='tensorflow' and  arr.shape[0] in [3, 4]):
+        elif (_backend!='tensorflow' and  arr.shape[0] in [3, 4] and arr.shape[2] not in [3, 4]):
+            arr = arr.transpose([1, 2, 0])
+        elif _backend in ['pytorch', 'cntk'] and arr.ndim == 3 and arr.shape[0] in [3, 4] and arr.shape[2] not in [3, 4]:
             arr = arr.transpose([1, 2, 0])
         else:
             raise ValueError('3d image should be 3 or 4 channel. Got {} channel.'.format(arr.shape[0]))
@@ -171,7 +175,7 @@ def mask2array(img):
         arr = np.ascontiguousarray(arr)
     return arr
 
-def array2mask(arr):
+def array2mask(arr:np.ndarray):
     '''
 
     Parameters
@@ -183,12 +187,13 @@ def array2mask(arr):
 
     '''
     # confirm back to numpy
-    arr = to_numpy(arr)
     if arr.ndim not in [2, 3]:
         raise ValueError('image should be 2 or 3 dimensional. Got {} dimensions.'.format(arr.ndim))
     mode = None
+    if arr.max()==1:
+        arr=arr*255
     if arr.ndim == 2:
-        arr = np.expand_dims(arr, 2)
+        #arr = np.expand_dims(arr, 2)
         mode = 'L'
     elif arr.ndim == 3:
         if arr.shape[3] in [3, 4] and arr.shape[0] not in [3, 4]:
