@@ -1,28 +1,37 @@
 from __future__ import absolute_import
 from __future__ import division
 from __future__ import print_function
+
+import collections
+import inspect
+
 import matplotlib
 import matplotlib.pyplot as plt
 from copy import copy, deepcopy
 from collections import deque
 import six
+import random
 import logging
+import uuid
 import os
+import copy
 import sys
 import re
 import warnings
-from collections import OrderedDict
+from collections import defaultdict
 from functools import partial,wraps,update_wrapper
 from itertools import islice
 import operator
 import torch
 import torch.nn as nn
+import torch.onnx
 import torchvision
-from collections import OrderedDict
-import numpy as np
-from .common import to_list
 
-__all__ = ['to_numpy','to_tensor','print_network','plot_tensor_grid','summary','calculate_flops','Modulex','Sequential','Input','ShortcutContainer','ConcatContainer','get_device','load']
+import numpy as np
+from .common import to_list,addindent,camel2snake,snake2camel,unpack_singleton,enforce_singleton,OrderedDict
+from .pytorch_ops import *
+
+__all__ = ['to_numpy','to_tensor','print_network','plot_tensor_grid','summary','calculate_flops', 'Layer', 'Sequential', 'Input', 'get_device', 'load','Combine','ReplayBuffer','check_keys','try_map_args_and_call','get_signature']
 
 version=torch.__version__
 sys.stderr.write('Pytorch version:{0}.\n'.format(version))
@@ -38,7 +47,11 @@ if torch.cuda.is_available() :
 def get_device():
     return  _device
 
-load=torch.load
+def load(model,path):
+    checkpoint = torch.load(path)
+    model.load_state_dict(checkpoint['state_dict'])
+    return model
+
 
 
 import sys
@@ -60,26 +73,39 @@ def to_numpy(x) -> np.ndarray:
     if isinstance(x, np.ndarray):
         return x
     elif isinstance(x, torch.Tensor):
-        return x.cpu().detach().numpy()
-    elif isinstance(x, (list, tuple, int, float)):
+        return x.clone().cpu().detach_().numpy()
+    elif isinstance(x, list):
         return np.array(x)
+    elif isinstance(x, tuple):
+        return np.array(list(x))
+    elif  'int' in str(type(x)) or  'float' in str(type(x)):
+        return np.array([x])
     else:
         raise ValueError("Unsupported type")
 
-def to_tensor(x, dtype=torch.float32) -> torch.Tensor:
+def to_tensor(x, dtype=torch.float32,requires_grad=None) -> torch.Tensor:
     if isinstance(x,  torch.Tensor):
         if dtype is not None:
             x = x.type(dtype)
         x=x.to(_device)
+        if requires_grad is not None:
+            x.requires_grad=requires_grad
         return x
-
-    if isinstance(x, (list, tuple)):
-        x = np.ndarray(x)
-
-    if isinstance(x, np.ndarray):
+    elif isinstance(x, int):
+        return torch.tensor(x).int().to(_device) if requires_grad is None else torch.tensor(x,requires_grad=requires_grad).int().to(_device)
+    elif isinstance(x, float):
+        return torch.tensor(x).float().to(_device) if requires_grad is None else torch.tensor(x,requires_grad=requires_grad).float().to(_device)
+    elif isinstance(x, (list, tuple)):
+        if isinstance(x[0],int):
+            x =torch.tensor(x).int() if requires_grad is None else torch.tensor(x,requires_grad=requires_grad).int()
+        else:
+            x=torch.tensor(x).float() if requires_grad is None else torch.tensor(x,requires_grad=requires_grad).float()
+        x = x.to(_device)
+        return x
+    elif isinstance(x, np.ndarray):
         npdtype=x.dtype
-        x = torch.from_numpy(x)
-        if 'int64' in str(npdtype):
+        x = torch.tensor(x) if requires_grad is None else torch.tensor(x,requires_grad=requires_grad)
+        if 'int' in str(npdtype):
             x = x.type(torch.int64)
         else:
             x = x.type(dtype)
@@ -89,30 +115,161 @@ def to_tensor(x, dtype=torch.float32) -> torch.Tensor:
         raise ValueError("Unsupported input type" + str(type(x)))
 
 
-class Modulex(nn.Module):
+
+_UID_PREFIXES = defaultdict(int)
+
+def get_uid(prefix=''):
+    _UID_PREFIXES[prefix] += 1
+    return _UID_PREFIXES[prefix]
+
+# class InputSpec(torch.IntTensor):
+#   """Specifies the ndim, dtype and shape of every input to a layer.
+#   Every layer should expose (if appropriate) an `input_spec` attribute:
+#   a list of instances of InputSpec (one per input tensor).
+#   A None entry in a shape is compatible with any dimension,
+#   a None shape is compatible with any shape.
+#   Arguments:
+#       dtype: Expected DataType of the input.
+#       shape: Shape tuple, expected shape of the input
+#           (may include None for unchecked axes).
+#       ndim: Integer, expected rank of the input.
+#       max_ndim: Integer, maximum rank of the input.
+#       min_ndim: Integer, minimum rank of the input.
+#       axes: Dictionary mapping integer axes to
+#           a specific dimension value.
+#   """
+#
+#   def __init__(self, input_tensor_shape):
+#
+#     @property
+#     def ndim(self):
+#         return len(self.size())
+#
+#     @property
+#     def shape(self):
+#         return tuple(self.tolist())
+#
+#   def __repr__(self):
+#     spec = [('dtype=' + str(self.dtype)) if self.dtype else '',
+#             ('shape=' + str(self.shape)) if self.shape else '',
+#             ('ndim=' + str(self.ndim)) if self.ndim else '',
+#             ('max_ndim=' + str(self.max_ndim)) if self.max_ndim else '',
+#             ('min_ndim=' + str(self.min_ndim)) if self.min_ndim else '',
+#             ('axes=' + str(self.axes)) if self.axes else '']
+#     return 'InputSpec(%s)' % ', '.join(x for x in spec if x)
+
+
+class Layer(nn.Module):
     """
     Applies a lambda function on forward()
     Args:
         lamb (fn): the lambda function
     """
 
-    def __init__(self):
-        super(Modulex, self).__init__()
-        self._is_built= False
-        self._build_hook=None
-        self._input_shape=None
-        self.output_shape = None
+    def __init__(self,**kwargs):
+        super(Layer, self).__init__()
+        self._built= False
+
+        self.register_buffer('_input_shape', None)
+        self.register_buffer('_output_shape',None)
+        self._input_shape = None
+        self._output_shape = None
+
+        self._output_tensor = None
+        self.is_track_output=False
+        self.signature=None
+
+        # self._input_shape=None
+        # self._output_shape = None
         self.input_filters =None
-        self.batch_size=-1
 
+
+        prefix = self.__class__.__name__
+        self.defaultname = camel2snake(prefix) + '_' + str(get_uid(prefix))
+
+        self._name = kwargs.get('name')
         self.dump_patches = True
-        self._fn = []
 
+        self.register_buffer('uuid',torch.tensor(uuid.uuid4().node))
+        self.uuid=torch.tensor(uuid.uuid4().node)
 
-        self.regist_shape_infer_hook()
+        self._nodes = OrderedDict()
+        self._nodes[self.uuid]=self
 
     @property
-    def trainable_variables(self) -> List[nn.Parameter]:
+    def name(self):
+        return self._name if self._name is not None and len(self._name)>0 else self.defaultname
+
+    @name.setter
+    def name(self,value):
+        self._name=value
+
+
+    @property
+    def nodes(self):
+        return self._nodes
+
+    @nodes.setter
+    def nodes(self,value):
+        if self._nodes!=value:
+            self._nodes=value
+            for mod in self.modules():
+                mod._nodes=value
+
+    def add_module(self, name, module):
+        r"""Adds a child module to the current module.
+
+        The module can be accessed as an attribute using the given name.
+
+        Args:
+            name (string): name of the child module. The child module can be
+                accessed from this module using the given name
+            module (Module): child module to be added to the module.
+        """
+        if not isinstance(module, (nn.Module,Layer)) and module is not None:
+            raise TypeError("{} is not a Module subclass".format(
+                torch.typename(module)))
+        elif not isinstance(name, torch._six.string_classes):
+            raise TypeError("module name should be a string. Got {}".format(torch.typename(name)))
+        elif hasattr(self, name) and name not in self._modules:
+            raise KeyError("attribute '{}' already exists".format(name))
+        elif '.' in name:
+            raise KeyError("module name can't contain \".\"")
+        elif name == '':
+            raise KeyError("module name can't be empty string \"\"")
+        nodes=self._nodes.copy()
+        for k,v in module.nodes.items():
+            nodes[k]=v
+        self._modules[name] = module
+        self.nodes=nodes
+        for mod in self.modules():
+            mod.nodes = nodes
+
+
+
+    def build(self, input_shape):
+        pass  #pass if no need shape infer
+
+    def compute_output_shape(self, input_shape):
+        """Computes the output shape of the layer.
+        Assumes that the layer will be built
+        to match that input shape provided.
+        # Arguments
+            input_shape: Shape tuple (tuple of integers)
+                or list of shape tuples (one per output tensor of the layer).
+                Shape tuples can include None for free dimensions,
+                instead of an integer.
+        # Returns
+            An output shape tuple.
+        """
+        if not self._built:
+            self.input_shape=input_shape
+        return self.output_shape
+
+
+
+    @property
+    def trainable_weights(self) -> List[nn.Parameter]:
         r"""The list of trainable variables (parameters) of the module.
         Parameters of this module and all its submodules are included.
         .. note::
@@ -122,62 +279,206 @@ class Modulex(nn.Module):
         """
         return [x for x in self.parameters() if x.requires_grad]
 
+    @property
+    def non_trainable_weights(self) -> List[nn.Parameter]:
+        r"""The list of trainable variables (parameters) of the module.
+        Parameters of this module and all its submodules are included.
+        .. note::
+            The list returned may contain duplicate parameters (e.g. output
+            layer shares parameters with embeddings). For most usages, it's not
+            necessary to ensure uniqueness.
+        """
+        return [x for x in self.parameters() if x.requires_grad==False]
+
+    @property
+    def weights(self):
+        return list(self.parameters())
+
+    def get_weights(self):
+        return list(self.parameters())
+
+
+    def set_weights(self, weights):
+        params = self.weights
+        if not params:
+            return
+        weight_value_tuples = []
+        param_values = [w.data for w in params]
+        for pv, p, w in zip(param_values, params, weights):
+            if pv.shape != w.shape:
+                raise ValueError('Layer weight shape ' + str(pv.shape) + ' not compatible with '
+                                                                         'provided weight shape ' + str(w.shape))
+            weight_value_tuples.append((p, w))
+        for p, w in weight_value_tuples:
+            p.data = w.data
+
+
+    @property
+    def trainable(self):
+        if len(self.weights)==0:
+            return False
+        elif len(self.weights)>0:
+            for k,v in self._parameters.items():
+                if v is not None and v.requires_grad==False:
+                    return False
+            else:
+                return True
+    @trainable.setter
+    def trainable(self,value:bool):
+        n=0
+        for name, para in self.named_parameters():
+            para.requires_grad = value
+            n+=np.prod(list(para.size()))
+        if value:
+            print('{0} parameters have set trainable'.format(n))
+        else:
+            print('{0} parameters have set untrainable'.format(n))
+
+
+
+    def get_config(self):
+        config = {'name': self.name,
+                  'trainable': self.trainable}
+
+        if hasattr(self, 'batch_input_shape'):
+            config['batch_input_shape'] = self.batch_input_shape
+
+        if hasattr(self, 'dtype'):
+            config['dtype'] = self.dtype
+
+        return config
+
     @property
     def device(self):
         return torch.device("cuda" if torch.cuda.is_available() else "cpu")
 
     @property
-    def is_built(self):
-        return self._is_built
+    def built(self):
+        return self._built
 
     @property
     def input_shape(self):
         return self._input_shape
+    #
+    # @property
+    # def input_spec(self):
+    #     return self._input_spec
+    #
+    # @input_spec.setter
+    # def input_spec(self, value):
+    #     for v in nest.flatten(value):
+    #         if v is not None and not isinstance(v, InputSpec):
+    #             raise TypeError('Layer input_spec must be an instance of InputSpec. '
+    #                             'Got: {}'.format(v))
+    #     self._input_spec = value
+
+    # @property
+    # def input(self):
+    #     """Retrieves the input tensor(s) of a layer.
+    #     Only applicable if the layer has exactly one input,
+    #     i.e. if it is connected to one incoming layer.
+    #     Returns:
+    #         Input tensor or list of input tensors.
+    #     Raises:
+    #       RuntimeError: If called in Eager mode.
+    #       AttributeError: If no inbound nodes are found.
+    #     """
+    #     return self._input_tensors
+
+    @property
+    def output(self):
+        """Retrieves the output tensor(s) of a layer.
+        Only applicable if the layer has exactly one output,
+        i.e. if it is connected to one incoming layer.
+        Returns:
+          Output tensor or list of output tensors.
+        Raises:
+          AttributeError: if the layer is connected to more than one incoming
+            layers.
+          RuntimeError: if called in Eager mode.
+        """
+        return self._output_tensor
+
+
+
 
     @input_shape.setter
     def input_shape(self, value):
-        if self._is_built == False:
-            if isinstance(value,list):
-                self._input_shape = value
-            elif isinstance(value,tuple):
-                self._input_shape =to_list(value)
-            elif isinstance(value, torch.Tensor) and value.dim()<=1:
-                self._input_shape = to_numpy(value.int()).tolist()
-            elif isinstance(value, np.ndarray) and value.ndim<=1:
-                self._input_shape = value.astype(np.uint8).tolist()
+        if isinstance(value, (list,tuple)) and len(value)>0:
+            if isinstance(value[0], torch.Size):
+                value=to_tensor(to_numpy(list(value[0]))).int()
+            elif isinstance(value[0], torch.Tensor):
+                value = value[0].int()
             else:
-                raise ValueError('not valid input_shape')
+                value=to_tensor(to_numpy(list(value)))
+        elif isinstance(value, torch.Size):
+            value=to_tensor(to_numpy(list(value))).int()
+        elif isinstance(value, torch.Tensor):
+            value=value.int()
+        elif isinstance(value, np.ndarray) and value.ndim <= 1:
+            value=torch.tensor(value.astype(np.uint8))
+        else:
+            raise ValueError('not valid input_shape')
 
 
-            self.input_filters = self._input_shape[0]
-            self.build_once(self._input_shape)
-            batch_input_shape=self._input_shape.copy()
-            batch_input_shape.insert(0,2)
-            x = torch.from_numpy(np.random.standard_normal(batch_input_shape).astype(np.float32)).to(self.device)
-            out=self.forward(x)
-            self._is_built=True
-            self.output_shape=to_list(out.size()[2:])
+        if self._built == False :
+            self._input_shape =value
+            self.input_filters = -1 if self._input_shape.ndim==0 else int(self._input_shape[0])
+            self.build(self._input_shape)
+            self._built = True
 
 
+        elif self._input_shape is not None and self._input_shape.tolist()==to_list(value):
+            'input_shape is already assigned, and shape is the same.'
+            pass
 
+    @property
+    def output_shape(self):
+        return  to_tensor(to_numpy(list(self._output_tensor.size()[1:]))) if self._output_shape is None else self._output_shape
 
-
-    def regist_shape_infer_hook(self):
-        def hook(module, input):
-            if module._is_built == False and isinstance(input,list) and callable(getattr(module, 'build_once')):
-                module.input_shape = [inp.size()[1:] for inp in input]
-            elif module._is_built == False and callable(getattr(module, 'build_once')):
-                module.input_shape=to_numpy(input[0]).shape[1:]
-
-        if (isinstance(self, Modulex)  and not isinstance(self, Sequential) and self._is_built== False):
-            self._build_hook=self.register_forward_pre_hook(hook)
+    @output_shape.setter
+    def output_shape(self, value):
+        if isinstance(value, (list, tuple)) and len(value) > 0:
+            if isinstance(value[0], torch.Size):
+                value = to_tensor(to_numpy(list(value[0]))).int()
+            elif isinstance(value[0], torch.Tensor):
+                value = value[0].int()
+            else:
+                value = to_tensor(to_numpy(list(value)))
+        elif isinstance(value, torch.Size):
+            value = to_tensor(to_numpy(list(value))).int()
+        elif isinstance(value, torch.Tensor):
+            value = value.int()
+        elif isinstance(value, np.ndarray) and value.ndim <= 1:
+            value = torch.tensor(value.astype(np.uint8))
+        else:
+            self._output_shape = value
+        self._output_shape = value
 
     def reset_parameters(self):
         pass
-    def build_once(self,input_shape ):
-        pass  #pass if no need shape infer
-    def shape_infer(self,input_shape):
-        self.input_shape=input_shape
+
+    def copy(self):
+        return copy.deepcopy(self)
+
+    def save_onnx(self,file_path=''):
+        input_shape=self.input_shape.copy()
+        input_shape.insert(0,1)
+        x = torch.randn(*input_shape, requires_grad=True)
+        torch_out = self(x)
+
+        # Export the model
+        torch.onnx.export(self,  # model being run
+                          x,  # model input (or a tuple for multiple inputs)
+                          file_path,  # where to save the model (can be a file or file-like object)
+                          export_params=True,  # store the trained parameter weights inside the model file
+                          opset_version=10,  # the ONNX version to export the model to
+                          do_constant_folding=True,  # whether to execute constant folding for optimization
+                          input_names=['input'],  # the model's input names
+                          output_names=['output'],  # the model's output names
+                          dynamic_axes={'input': {0: 'batch_size'},  # variable lenght axes
+                                        'output': {0: 'batch_size'}})
+
     def __call__(self, *input, **kwargs):
         for hook in self._forward_pre_hooks.values():
             result = hook(self, input)
@@ -185,19 +486,21 @@ class Modulex(nn.Module):
                 if not isinstance(result, tuple):
                     result = (result,)
                 input = result
-        #self._build_hook = self.register_forward_pre_hook(hook)
-        if self._build_hook is not None:
-            self._build_hook.remove()
-            self._build_hook=None
-            self._is_built=True
-        if isinstance(input[0],Input):
-            inp=input[0]
-            inp._fn.append(self)
-            return inp
+        if self._built==False :
+            self.input_shape =input[0].size()[1:] if isinstance(input[0],torch.Tensor) else torch.tensor(-1)
+            self.build(self.input_shape)
         if torch._C._get_tracing_state():
             result = self._slow_forward(*input, **kwargs)
         else:
             result = self.forward(*input, **kwargs)
+            result=unpack_singleton(result)
+            if isinstance(result,torch.Tensor) :
+                if self._output_shape is None:
+                    self.output_shape=result.size()[1:]
+                #self._output_tensor = result.clone().cpu().detach_()
+
+
+
         for hook in self._forward_hooks.values():
             hook_result = hook(self, input, result)
             if hook_result is not None:
@@ -217,38 +520,53 @@ class Modulex(nn.Module):
                     grad_fn.register_hook(wrapper)
         return result
 
+    def __repr__(self):
+        # We treat the extra repr like the sub-module, one item per line
+        extra_lines = []
+        extra_repr = self.extra_repr()
+        # empty string will be split into list ['']
+        if extra_repr:
+            extra_lines = extra_repr.split('\n')
+        child_lines = []
+        for key, module in self._modules.items():
+            mod_str = repr(module)
+            mod_str = addindent(mod_str, 2)
+            child_lines.append('(' + key + '): ' + mod_str)
+        lines = extra_lines + child_lines
 
-class Input(Modulex):
-    def __init__(self, inputshape: (list, tuple) = None, batch_shape: (list, tuple) = None,
-                 inputtensor: torch.tensor = None):
-        super().__init__()
-        if inputtensor is not None:
-            self.inputtensor = inputtensor
-            self.batch_shape = list(to_numpy(inputtensor.size()))
-            self.inputshape = self.batch_shape[1:]
-        else:
-            if self.batch_shape is not None:
-                if isinstance(batch_shape, tuple):
-                    self.batch_shape = [int(batch_shape[i]) for i in range(len(batch_shape))]
-                else:
-                    self.batch_shape = batch_shape
-                self.inputshape = self.batch_shape[1:]
+        main_str = self._get_name() + '('
+        if lines:
+            # simple one-liner info, which most builtin Modules will use
+            if len(extra_lines) == 1 and not child_lines:
+                main_str += extra_lines[0]
             else:
-                if isinstance(inputshape, tuple):
-                    self.inputshape = [int(inputshape[i]) for i in range(len(inputshape))]
-                else:
-                    self.inputshape = inputshape
+                main_str += '\n  ' + '\n  '.join(lines) + '\n'
+
+        main_str += ')'
+        return main_str
+
+
+
+
+
+class Input(Layer):
+    def __init__(self, input_shape: (list, tuple,int) = None,name=''):
+        super().__init__()
+        self.name=name
+        if isinstance(input_shape,int):
+            input_shape=input_shape,
+        self.input_shape=tuple(input_shape)
+        self._built=True
 
-                self.batch_shape = self.inputshape.copy()
-                self.batch_shape.insert(0, 1)
 
     def forward(self, x):
         if x is None:
-            return torch.tensor(np.zeros(self.batch_shape, dtype=np.float32))
+            return torch.rand(2,*self.input_shape)
         else:
             return x
 
-class Sequential(Modulex):
+
+class Sequential(Layer):
     r"""A sequential container.
     Modules will be added to it in the order they are passed in the constructor.
     Alternatively, an ordered dict of modules can also be passed in.
@@ -272,18 +590,68 @@ class Sequential(Modulex):
                 ]))
     """
 
-    def __init__(self, *args):
-        super(Sequential, self).__init__()
-        self._is_built = False
+    def __init__(self, *args,name=''):
+        super(Sequential, self).__init__(name=name)
+        self._built = False
 
         if len(args) == 1 and isinstance(args[0], OrderedDict):
             for key, module in args[0].items():
                 self.add_module(key, module)
+        elif len(args) == 1 and isinstance(args[0], (list,nn.ModuleList)):
+            for  idx, module in enumerate(args[0]):
+                self.add_module(str(idx), module[1])
         else:
             for idx, module in enumerate(args):
                 self.add_module(str(idx), module)
         self.to(self.device)
 
+    def build(self, input_shape):
+        if self._built==False and len(self._modules)>0:
+            self.__getitem__(0).input_shape=self.input_shape
+            self._built=True
+
+    def add_module(self, name, module):
+        r"""Adds a child module to the current module.
+
+        The module can be accessed as an attribute using the given name.
+
+        Args:
+            name (string): name of the child module. The child module can be
+                accessed from this module using the given name
+            module (Module): child module to be added to the module.
+        """
+        final_output=None
+        if len(self._modules)>0:
+            last_module=list(self._modules.values())[-1]
+            if last_module.built and last_module.output_shape is not None:
+                final_output=to_list(last_module.output_shape)
+
+        if not isinstance(module,Layer) and module is not None:
+            raise TypeError("{} is not a Module subclass".format(
+                torch.typename(module)))
+        elif not isinstance(name, torch._six.string_classes):
+            raise TypeError("module name should be a string. Got {}".format(
+                torch.typename(name)))
+        elif hasattr(self, name) and name not in self._modules:
+            raise KeyError("attribute '{}' already exists".format(name))
+        elif '.' in name:
+            raise KeyError("module name can't contain \".\"")
+        elif name == '':
+            raise KeyError("module name can't be empty string \"\"")
+        if final_output is not None:
+            module.input_shape=final_output
+        self._modules[name] = module
+
+
+    def sync_build(self):
+        input_shape=None
+        if self[:1] is Input:
+            input_shape=self[:1].input_shape
+        if input_shape is not None:
+            input_shape = list(input_shape)
+            input_shape.insert(0, 2)
+            data = torch.tensor(np.random.standard_normal(input_shape))
+            out = self(data)
 
 
     def _get_item_by_idx(self, iterator, idx):
@@ -297,7 +665,7 @@ class Sequential(Modulex):
 
     def __getitem__(self, idx):
         if isinstance(idx, slice):
-            return self.__class__(OrderedDict(list(self._modules.items())[idx]))
+            return self.__class__(list(self._modules.items())[idx])
         else:
             return self._get_item_by_idx(self._modules.values(), idx)
 
@@ -321,12 +689,15 @@ class Sequential(Modulex):
         keys = [key for key in keys if not key.isdigit()]
         return keys
 
-    def forward(self, x):
+    def forward(self, *x):
+        x = enforce_singleton(x)
         for module in self._modules.values():
             x = module(x)
         return x
 
-class ConcatContainer(Modulex):
+
+
+class Combine(Layer):
     r"""A sequential container.
     Modules will be added to it in the order they are passed in the constructor.
     Alternatively, an ordered dict of modules can also be passed in.
@@ -351,20 +722,21 @@ class ConcatContainer(Modulex):
     """
 
     def __init__(self, *args):
-        super(ConcatContainer, self).__init__()
-        self._is_built = False
-        self.axis = 1
+        super(Combine, self).__init__()
+        self._built = False
 
         if len(args) == 1 and isinstance(args[0], OrderedDict):
             for key, module in args[0].items():
                 self.add_module(key, module)
+        elif len(args) == 1 and isinstance(args[0], (list,nn.ModuleList)):
+            for  idx, module in enumerate(args[0]):
+                self.add_module(str(idx), module)
         else:
             for idx, module in enumerate(args):
                 self.add_module(str(idx), module)
         self.to(self.device)
 
 
-
     def _get_item_by_idx(self, iterator, idx):
         """Get the idx-th item of the iterator"""
         size = len(self)
@@ -396,98 +768,49 @@ class ConcatContainer(Modulex):
         return len(self._modules)
 
     def __dir__(self):
-        keys = super(ConcatContainer, self).__dir__()
+        keys = super(Combine, self).__dir__()
         keys = [key for key in keys if not key.isdigit()]
         return keys
 
-    def forward(self, x):
-        results=[]
+    def forward(self, *x):
+        outputs=[]
         for module in self._modules.values():
-            x1 = module(x)
-            results.append(x1)
-        return torch.cat(results,dim=1)
-
-class ShortcutContainer(Modulex):
-    r"""A sequential container.
-    Modules will be added to it in the order they are passed in the constructor.
-    Alternatively, an ordered dict of modules can also be passed in.
-
-    To make it easier to understand, here is a small example::
-
-        # Example of using Sequential
-        model = nn.Sequential(
-                  nn.Conv2d(1,20,5),
-                  nn.ReLU(),
-                  nn.Conv2d(20,64,5),
-                  nn.ReLU()
-                )
-
-        # Example of using Sequential with OrderedDict
-        model = nn.Sequential(OrderedDict([
-                  ('conv1', nn.Conv2d(1,20,5)),
-                  ('relu1', nn.ReLU()),
-                  ('conv2', nn.Conv2d(20,64,5)),
-                  ('relu2', nn.ReLU())
-                ]))
-    """
-
-    def __init__(self, *args):
-        super(ShortcutContainer, self).__init__()
-        self._is_built = False
-        self.axis = 1
-
-        if len(args) == 1 and isinstance(args[0], OrderedDict):
-            for key, module in args[0].items():
-                self.add_module(key, module)
-        else:
-            for idx, module in enumerate(args):
-                self.add_module(str(idx), module)
-        self.to(self.device)
+            outputs.append(module(*x))
+        return tuple(outputs)
 
 
+class ReplayBuffer:
+    def __init__(self, max_size=1000):
+        assert max_size > 0, "Empty buffer or trying to create a black hole. Be careful."
+        self.max_size = max_size
+        self.data = []
 
-    def _get_item_by_idx(self, iterator, idx):
-        """Get the idx-th item of the iterator"""
-        size = len(self)
-        idx = idx.__index__()
-        if not -size <= idx < size:
-            raise IndexError('index {} is out of range'.format(idx))
-        idx %= size
-        return next(islice(iterator, idx, None))
+    def push_and_pop(self, data):
+        to_return = []
+        keep_idx=random.choice(range(data.data.size(0)))
+        for i in range(data.data.size(0)):
+            element=data.data[i]
+            element = torch.unsqueeze(element, 0)
 
-    def __getitem__(self, idx):
-        if isinstance(idx, slice):
-            return self.__class__(OrderedDict(list(self._modules.items())[idx]))
-        else:
-            return self._get_item_by_idx(self._modules.values(), idx)
+            if len(self.data) > 10 and random.uniform(0, 1) > 0.7:
+                i = random.randint(0, self.max_size - 1)
+                to_return.append(self.data[i].clone())
+            else:
+                to_return.append(element)
 
-    def __setitem__(self, idx, module):
-        key = self._get_item_by_idx(self._modules.keys(), idx)
-        return setattr(self, key, module)
+            if 0<len(self.data) < self.max_size and keep_idx==i:
+                self.data.append(element)
+            elif len(self.data) == self.max_size and random.randint(0,10)%3==0 and keep_idx==i:
+                self.data[random.randint(0, self.max_size - 1)] = element
 
-    def __delitem__(self, idx):
-        if isinstance(idx, slice):
-            for key in list(self._modules.keys())[idx]:
-                delattr(self, key)
-        else:
-            key = self._get_item_by_idx(self._modules.keys(), idx)
-            delattr(self, key)
-
-    def __len__(self):
-        return len(self._modules)
-
-    def __dir__(self):
-        keys = super(ShortcutContainer, self).__dir__()
-        keys = [key for key in keys if not key.isdigit()]
-        return keys
-
-    def forward(self, x):
-        results=0
-        for module in self._modules.values():
-            x1 = module(x)
-            results=results+x1
-        return results
+        to_return=shuffle(torch.cat(to_return))
+        return to_return
 
+    def push_only(self, data):
+        element=random_choice(data)
+        element = torch.unsqueeze(element, 0)
+        if len(self.data) < self.max_size:
+            self.data.append(element)
 
 
 def print_network(net, verbose=False):
@@ -554,14 +877,13 @@ def summary(model, input_size, batch_size=-1, device="cuda"):
             class_name = str(module.__class__).split(".")[-1].split("'")[0]
             module_idx = len(summary)
 
-            m_key = "%s-%i" % (class_name, module_idx + 1)
+            m_key =module.name if hasattr(module,'name') else camel2snake(module.__class__.__name__) + '_' + str(get_uid(module.__class__.__name__))
             summary[m_key] = OrderedDict()
             summary[m_key]["input_shape"] = list(input[0].size())
             summary[m_key]["input_shape"][0] = batch_size
             if isinstance(output, (list, tuple)):
                 summary[m_key]["output_shape"] = [
-                    [-1] + list(o.size())[1:] for o in output
-                ]
+                    [-1] + list(o.size())[1:] for o in output ]
             else:
                 summary[m_key]["output_shape"] = list(output.size())
                 summary[m_key]["output_shape"][0] = batch_size
@@ -625,11 +947,13 @@ def summary(model, input_size, batch_size=-1, device="cuda"):
     # remove these hooks
     for h in hooks:
         h.remove()
+    layer_width=(max([len(layer) for layer in summary])//5+1)*5
+
 
     print("--------------------------------------------------------------------------------------------------------------------------------")
-    line_new = "{:>40} {:>30}  {:>30} {:>15}  {:>15}  {:>15}".format("Layer (type)", "Output Shape","Weight Shape","Bias Shape", "Param #", "FLOPS #")
+    line_new = "{0:^45s} {1:^20s}  {2:^20s} {3:^8s}  {4:^8s}  {5:^12s}".format("Layer (type)", "Output Shape","Weight ","Bias", "Param #", "FLOPS #")
     print(line_new)
-    print("===========================================================================================================")
+    print("==============================================================================")
     total_params = 0
     total_output = 0
     trainable_params = 0
@@ -637,13 +961,13 @@ def summary(model, input_size, batch_size=-1, device="cuda"):
     macc=0
     for layer in summary:
         # input_shape, output_shape, trainable, nb_params
-        line_new = "{:>40} {:>30}  {:>30} {:>15}  {:>15}  {:>15}".format(
+        line_new = "{0:<45s} {1:<20s}  {2:<20s} {3:<8s}  {4:<8}  {5:<12}".format(
             layer,
             str(summary[layer]["output_shape"]),
             str(summary[layer]["weight"] if 'weight' in summary[layer] else ''),
             str(summary[layer]["bias"] if 'bias' in summary[layer] else ''),
-            "{0:,}".format(summary[layer]["nb_params"]),
-            "{0:,}".format(summary[layer]["flops"][0]),
+            summary[layer]["nb_params"],
+            summary[layer]["flops"][0]
         )
         total_params += summary[layer]["nb_params"]
         flops+= float(summary[layer]["flops"][0])
@@ -657,7 +981,7 @@ def summary(model, input_size, batch_size=-1, device="cuda"):
     # assume 4 bytes/number (float on cuda).
     total_input_size = abs(np.prod(input_size) * batch_size * 4. / (1024 ** 2.))
     total_output_size = abs(2. * total_output * 4. / (1024 ** 2.))  # x2 for gradients
-    total_params_size = abs(total_params.numpy() * 4. / (1024 ** 2.))
+    total_params_size = abs(total_params * 4. / (1024 ** 2.))
     total_size = total_params_size + total_output_size + total_input_size
 
     print("================================================================")
@@ -988,8 +1312,63 @@ def get_human_readable_count(number):
     index = num_groups - 1
     return '{int(number):,d} {labels[index]}'
 
+def check_keys(model, pretrained_state_dict):
+    ckpt_keys = set(pretrained_state_dict.keys())
+    model_keys = set(model.state_dict().keys())
+    used_pretrained_keys = model_keys & ckpt_keys
+    unused_pretrained_keys = ckpt_keys - model_keys
+    missing_keys = model_keys - ckpt_keys
+    print('Missing keys:{}'.format(len(missing_keys)))
+    print('Unused checkpoint keys:{}'.format(len(unused_pretrained_keys)))
+    print('Used keys:{}'.format(len(used_pretrained_keys)))
+    assert len(used_pretrained_keys) > 0, 'load NONE from pretrained checkpoint'
+    return True
 
 
+def get_signature(fn):
+    signature = OrderedDict()
+    parameters = list(inspect.signature(fn).parameters.values())
+    for k in parameters:
+        signature[k.name]= k.default if k.default !=inspect._empty else None
+    return signature
+
+
+def try_map_args_and_call(fn, data: OrderedDict,data_feed=None):
+    arg_map = OrderedDict()
+    if isinstance(fn,Layer) :
+        for arg in fn.signature.key_list:
+            if arg in data_feed:
+                arg_map[arg]=data[data_feed[arg]]
+            else:
+                raise ValueError('arg :{0} cannot mapping correctly!'.format(arg))
+        #print('arg_map',arg_map.key_list)
+        out=fn(*arg_map.value_list)
+        return out
+    elif hasattr(fn,'signature') and callable(fn):
+        for arg in fn.signature.key_list:
+            if arg in data:
+                arg_map[arg]=data[arg]
+            elif arg in data_feed:
+                arg_map[arg]=data[data_feed[arg]]
+            else:
+                raise ValueError('arg :{0} cannot mapping correctly!'.format(arg))
+        #print('arg_map', arg_map.key_list)
+        out=fn(*arg_map.value_list)
+        return out
+    elif  callable(fn):
+
+        args=get_signature(fn).key_list
+        for arg in args:
+            if arg in  data_feed:
+                arg_map[arg]=data[data_feed[arg]]
+            else:
+                arg_map[arg] = ''
+        #print('arg_map', arg_map.key_list)
+        out = fn(*arg_map.value_list)
+        return out
+    else:
+
+        print('uncomplete arg_map', arg_map.key_list)
 
 
 
