@@ -1,30 +1,61 @@
 import os
 import sys
 from typing import Tuple, Optional, Union
-
+from collections import Sized, Iterable
 import torch
 import torch.nn as nn
-
 import torch.nn.functional as F
 from torch.autograd import Variable
 from functools import partial
 import numpy as np
-from .common import get_session,addindent,get_time_suffix,get_class,PrintException
+from .common import *
 _device = torch.device("cuda" if torch.cuda.is_available() else "cpu")
-__all__ = ['argmax','reduce_max','reduce_min','reduce_mean','reduce_sum','expand_dims','meshgrid','element_cosine_distance','gram_matrix','get_rotation_matrix2d','warp_affine']
+__all__ = ['reshape','argmax','log_sum_exp','reduce_max','reduce_min','reduce_mean','reduce_sum','expand_dims','shuffle','random_choice','meshgrid','element_cosine_distance','gram_matrix','get_rotation_matrix2d','warp_affine','binary_crossentropy']
 
 
-def argmax(t:torch.Tensor,axis=1):
-    _, idx = t.max(dim=axis)
+
+
+
+
+
+def reshape(x,shape=None)-> torch.Tensor:
+    if shape is None:
+        return x
+    elif isinstance(shape,list):
+        return torch.reshape(x,shape)
+    elif isinstance(shape,tuple):
+        shape=to_list(shape)
+        return torch.reshape(x,shape)
+    else:
+        return x
+
+
+
+
+
+def log_sum_exp(x,axis=1)-> torch.Tensor:
+
+    x_max = x.data.max()
+    return torch.log(torch.sum(torch.exp(x-x_max), dim=axis, keepdim=True)) + x_max
+
+
+
+
+
+
+
+def argmax(x:torch.Tensor,axis=1)-> torch.Tensor:
+    if len(x.shape)>axis:
+         _, idx = x.max(dim=axis)
+    else:
+        _, idx = x.max()
     return idx
 
-
 def reduce_mean(t:torch.Tensor,axis=1,keepdim=False):
     return t.mean(dim=axis,keepdim=keepdim)
 def reduce_sum(t:torch.Tensor,axis=1,keepdim=False):
     return t.sum(dim=axis,keepdim=keepdim)
 
-
 def reduce_max(t:torch.Tensor,axis=1,keepdim=False):
     arr, idx = t.max(dim=axis,keepdim=keepdim)
     return arr
@@ -36,6 +67,14 @@ def reduce_min(t:torch.Tensor,axis=1,keepdim=False):
 def expand_dims(t:torch.Tensor,axis=0):
     return t.unsqueeze(axis)
 
+def shuffle(t:torch.Tensor):
+    order = np.random.shuffle(np.array(range(t.size(0))))
+    t[np.array(range(t.size(0)))] = t[order]
+    return t
+
+def random_choice(t:torch.Tensor):
+    idx = np.random.choice(np.array(range(t.size(0))))
+    return t[idx]
 
 def element_cosine_distance(v1, v2, axis=-1):
     reduce_dim = -1
@@ -44,7 +83,127 @@ def element_cosine_distance(v1, v2, axis=-1):
 
 
 
-def meshgrid(x, y, normalized_coordinates=True):
+def binary_crossentropy(target, output, from_logits=False):
+    if from_logits:
+        output = output.sigmoid_()
+    output = output.clamp_( epsilon(), 1.0 - epsilon())
+    output = -target * torch.log(output) - (1.0 - target) * torch.log(1.0 - output)
+    return output
+
+
+
+
+
+def torch_rot90_(x: torch.Tensor):
+    return x.transpose_(2, 3).flip(2)
+
+
+def torch_rot90(x: torch.Tensor):
+    return x.transpose(2, 3).flip(2)
+
+
+def torch_rot180(x: torch.Tensor):
+    return x.flip(2).flip(3)
+
+
+def torch_rot270(x: torch.Tensor):
+    return x.transpose(2, 3).flip(3)
+
+
+def torch_flipud(x: torch.Tensor):
+    """
+    Flip image tensor vertically
+    :param x:
+    :return:
+    """
+    return x.flip(2)
+
+
+def torch_fliplr(x: torch.Tensor):
+    """
+    Flip image tensor horizontally
+    :param x:
+    :return:
+    """
+    return x.flip(3)
+
+
+
+
+
+
+def pad_image_tensor(image_tensor: torch.Tensor, pad_size: int = 32):
+    """Pad input tensor to make it's height and width dividable by @pad_size
+
+    :param image_tensor: Input tensor of shape NCHW
+    :param pad_size: Pad size
+    :return: Tuple of output tensor and pad params. Second argument can be used to reverse pad operation of model output
+    """
+    rows, cols = image_tensor.size(2), image_tensor.size(3)
+    if (
+        isinstance(pad_size, Sized)
+        and isinstance(pad_size, Iterable)
+        and len(pad_size) == 2
+    ):
+        pad_height, pad_width = [int(val) for val in pad_size]
+    elif isinstance(pad_size, int):
+        pad_height = pad_width = pad_size
+    else:
+        raise ValueError(
+            "Unsupported pad_size: {pad_size}, must be either tuple(pad_rows,pad_cols) or single int scalar."
+        )
+
+    if rows > pad_height:
+        pad_rows = rows % pad_height
+        pad_rows = pad_height - pad_rows if pad_rows > 0 else 0
+    else:
+        pad_rows = pad_height - rows
+
+    if cols > pad_width:
+        pad_cols = cols % pad_width
+        pad_cols = pad_width - pad_cols if pad_cols > 0 else 0
+    else:
+        pad_cols = pad_width - cols
+
+    if pad_rows == 0 and pad_cols == 0:
+        return image_tensor, (0, 0, 0, 0)
+
+    pad_top = pad_rows // 2
+    pad_btm = pad_rows - pad_top
+
+    pad_left = pad_cols // 2
+    pad_right = pad_cols - pad_left
+
+    pad = [pad_left, pad_right, pad_top, pad_btm]
+    image_tensor = torch.nn.functional.pad(image_tensor, pad)
+    return image_tensor, pad
+
+
+def unpad_image_tensor(image_tensor, pad):
+    pad_left, pad_right, pad_top, pad_btm = pad
+    rows, cols = image_tensor.size(2), image_tensor.size(3)
+    return image_tensor[..., pad_top : rows - pad_btm, pad_left : cols - pad_right]
+
+
+def unpad_xyxy_bboxes(bboxes_tensor: torch.Tensor, pad, dim=-1):
+    pad_left, pad_right, pad_top, pad_btm = pad
+    pad = torch.tensor(
+        [pad_left, pad_top, pad_left, pad_top], dtype=bboxes_tensor.dtype
+    ).to(bboxes_tensor.device)
+
+    if dim == -1:
+        dim = len(bboxes_tensor.size()) - 1
+
+    expand_dims = list(set(range(len(bboxes_tensor.size()))) - {dim})
+    for i, dim in enumerate(expand_dims):
+        pad = pad.unsqueeze(dim)
+
+    return bboxes_tensor - pad
+
+
+
+
+def meshgrid(x, y, normalized_coordinates=False):
     '''Return meshgrid in range x & y.
 
     Args:
@@ -80,9 +239,7 @@ def meshgrid(x, y, normalized_coordinates=True):
         xs = torch.linspace(-1, 1, x, device=_device, dtype=torch.float)
         ys = torch.linspace(-1, 1, y, device=_device, dtype=torch.float)
 
-
-    base_grid=torch.stack(torch.meshgrid([xs, ys])).transpose(1, 2)
-    return torch.unsqueeze(base_grid, dim=0).permute(0, 2, 3, 1)
+    return torch.stack(torch.meshgrid([xs, ys]))
 
 
 def gram_matrix(input):
