@@ -26,7 +26,7 @@ import hashlib
 import glob
 import six
 from scipy.io import loadmat
-
+from ..backend.common import OrderedDict,PrintException
 from six.moves.urllib.error import HTTPError
 from six.moves.urllib.error import URLError
 from six.moves.urllib.request import urlopen
@@ -107,10 +107,12 @@ def download_file(src, dirname, filename, desc=''):
     if os.path.exists(os.path.join(dirname, filename)) and _h != {} and _h.get('is_downloaded', False) == True:
         print('archive file is already existing, donnot need download again.')
     else:
+        if os.path.exists(os.path.join(dirname, filename)):
+            os.remove(os.path.join(dirname, filename))
         error_msg = 'URL fetch failure on {}: {} -- {}'
         try:
             try:
-                with TqdmProgress(unit='B', unit_scale=True, miniters=10, desc=desc) as t:  # all optional kwargs
+                with TqdmProgress(unit='B', unit_scale=True,  leave=True,miniters=10, desc=desc) as t:  # all optional kwargs
                     urlretrieve(src, filename=os.path.join(dirname, filename), reporthook=t.update_to, data=None)
                     _write_h(dirname, True, False)
             except HTTPError as e:
@@ -152,18 +154,84 @@ def download_file_from_google_drive(file_id, dirname, filename=None, md5=None):
     if os.path.exists(os.path.join(dirname, filename)) and _h!={} and  _h.get('is_downloaded', False)==True:
         print('archive file is already existing, donnot need download again.')
     else:
-        session = requests.Session()
-        response = session.get(url, params={'id': file_id}, stream=True)
-        token = _get_confirm_token(response)
+        with TqdmProgress(unit='B', unit_scale=True, miniters=10, desc='') as t:  # all optional kwargs
+            try:
+                session = requests.Session()
+                response = session.get(url, params={'id': file_id}, stream=True)
+                token = _get_confirm_token(response)
 
-        if token:
-            params = {'id': file_id, 'confirm': token}
-            response = session.get(url, params=params, stream=True)
+                if token:
+                    params = {'id': file_id, 'confirm': token}
+                    response = session.get(url, params=params, stream=True)
 
-        _save_response_content(response, fpath)
-        _write_h(dirname,True,False)
+                _save_response_content(response, fpath)
+                _write_h(dirname,True,False)
+                return True
+            except:
+                PrintException()
+                return False
+    return False
 
 
+def download_model_from_google_drive(file_id, dirname, filename=None, md5=None):
+    """Download a Google Drive file from  and place it in root.
+
+    Args:
+        file_id (str): id of file to be downloaded
+        root (str): Directory to place downloaded file in
+        filename (str, optional): Name to save the file under. If None, use the id of the file.
+        md5 (str, optional): MD5 checksum of the download. If None, do not check
+    """
+    # Based on https://stackoverflow.com/questions/38511444/python-download-files-from-google-drive-using-url
+    import requests
+    url = "https://docs.google.com/uc?export=download"
+
+    if not filename:
+        filename = file_id
+    fpath = os.path.join(dirname, filename)
+    if os.path.exists(os.path.join(dirname, 'models_md5.json')):
+        os.remove(os.path.join(dirname, 'models_md5.json'))
+
+    isload=download_file_from_google_drive('1AujFsfyzyYUcUrNI_GT1dvNrHaXIQkGz', dirname, 'models_md5.json')
+    models_md5 = OrderedDict()
+    need_download = True
+    if isload==False:
+        if os.path.exists(os.path.join(dirname, filename)):
+            stderr.write('download error, use local exsiting model')
+        else:
+            stderr.write('sorry, download failed, cannot get the pretrained model.')
+    else:
+        need_download=True
+        if os.path.exists(os.path.join(dirname, 'models_md5.json')):
+            with open(os.path.join(dirname, 'models_md5.json')) as f:
+                models_md5 = json.load(f)
+
+        # if os.path.exists(os.path.join(dirname, filename)):
+        #     print('archive file is already existing, donnot need download again.')
+
+        if os.path.exists(os.path.join(dirname, filename)) :
+            if check_integrity(os.path.join(dirname, filename), models_md5[filename]):
+                need_download=False
+                print('model file is already existing, donnot need download again.')
+            else:
+                print('Your pretrained model has newer version, will you want to update it?')
+                ans = input('(Y/N) << ').lower()
+                if ans in ['yes', 'y']:
+                    os.remove(os.path.join(dirname, filename))
+                else:
+                    need_download = False
+
+        if need_download:
+            with TqdmProgress(unit='B', unit_scale=True, miniters=10, desc='') as t:  # all optional kwargs
+                session = requests.Session()
+                response = session.get(url, params={'id': file_id}, stream=True)
+                token = _get_confirm_token(response)
+
+                if token:
+                    params = {'id': file_id, 'confirm': token}
+                    response = session.get(url, params=params, stream=True)
+                _save_response_content(response, fpath)
+
 def _get_confirm_token(response):
     for key, value in response.cookies.items():
         if key.startswith('download_warning'):
