@@ -4,23 +4,24 @@ from __future__ import print_function
 from pydoc import locate
 import six
 from functools import partial
+import inspect
 import math
-
+import string
 import torch
 import torch.nn as nn
 import torch.nn.functional as F
 import torch.nn.modules.activation as af
 from torch.nn.parameter import Parameter
-from ..backend.common import get_function,camel2snake
-from ..backend.pytorch_backend import Modulex
-__all__ = ['Identity','Sigmoid','Tanh','Relu','Relu6','LeakyRelu','LeakyRelu6','SmoothRelu','PRelu','Swish','Elu','HardSigmoid','HardSwish','Selu','LecunTanh','SoftSign','SoftPlus','HardTanh','Logit','LogLog','Mish','Softmax','BertGELU','GPTGELU','identity','sigmoid','tanh','relu','relu6','leaky_relu','leaky_relu6','smooth_relu','p_relu','swish','elu','hard_sigmoid','hard_swish','selu','lecun_tanh','soft_sign','soft_plus','hard_tanh','logit','log_log','mish','softmax','bert_gelu','gpt_gelu','get_activation']
+from ..backend.common import get_function,get_class,camel2snake,enforce_singleton
+from ..backend.pytorch_backend import Layer
+__all__ = ['Identity','Sigmoid','Tanh','Relu','Relu6','LeakyRelu','LeakyRelu6','SmoothRelu','PRelu','Swish','Elu','HardSigmoid','HardSwish','Selu','LecunTanh','SoftSign','SoftPlus','HardTanh','Logit','LogLog','Mish','Softmax','BertGelu','GptGelu','identity','sigmoid','tanh','relu','relu6','leaky_relu','leaky_relu6','smooth_relu','p_relu','swish','elu','hard_sigmoid','hard_swish','selu','lecun_tanh','soft_sign','soft_plus','hard_tanh','logit','log_log','mish','softmax','bert_gelu','gpt_gelu','get_activation']
 
 '''
 '''
-class Identity(Modulex):
-    def __init__(self, ):
-        super(Identity, self).__init__()
-        self._is_built = True
+class Identity(Layer):
+    def __init__(self, name=''):
+        super(Identity, self).__init__(name=name)
+        
     def forward(self, x):
         return x
 
@@ -29,10 +30,11 @@ class Identity(Modulex):
 def identity(x):
     return x
 
-class Relu(Modulex):
-    def __init__(self, ):
-        super(Relu, self).__init__()
-    def forward(self, x):
+class Relu(Layer):
+    def __init__(self, name=''):
+        super(Relu, self).__init__(name=name)
+    def forward(self, *x):
+        x = enforce_singleton(x)
         return relu(x)
 
 '''relu activation function 
@@ -41,46 +43,50 @@ def relu(x):
     return torch.relu(x)
 
 
-class Relu6(Modulex):
+class Relu6(Layer):
     def __init__(self, ):
         super(Relu6, self).__init__()
-    def forward(self, x):
+    def forward(self, *x):
+        x = enforce_singleton(x)
         return relu6(x)
 '''relu6 activation function 
 '''
 def relu6(x):
     return F.relu6(x)
 
-class LeakyRelu(Modulex):
+class LeakyRelu(Layer):
     def __init__(self, ):
         super(LeakyRelu, self).__init__()
-    def forward(self, x):
+    def forward(self, *x):
+        x = enforce_singleton(x)
         return leaky_relu(x)
 
 '''leaky_relu activation function 
 '''
 def leaky_relu(x):
-    return F.leaky_relu(x,negative_slope=0.1)
+    return F.leaky_relu(x,negative_slope=0.2)
 
 
-class LeakyRelu6(Modulex):
+class LeakyRelu6(Layer):
     def __init__(self):
         super(LeakyRelu6, self).__init__()
-        self._is_built = True
-    def forward(self, x):
+        
+    def forward(self, *x):
+        x = enforce_singleton(x)
         return leaky_relu6(x)
 
 '''leaky_relu6 activation function 
 '''
 def leaky_relu6(x):
-    return torch.clamp(F.leaky_relu(x,negative_slope=0.1),-6,6)
+    return torch.clamp(F.leaky_relu(x,negative_slope=0.2),-6,6)
 
 
-class SmoothRelu(Modulex):
+class SmoothRelu(Layer):
     def __init__(self):
         super(SmoothRelu, self).__init__()
-        self._is_built = True
-    def forward(self, x):
+        
+    def forward(self, *x):
+        x = enforce_singleton(x)
         return smooth_relu(x)
 
 
@@ -93,14 +99,16 @@ def smooth_relu(x):
 
 '''PRelu activation function Layer
 '''
-class PRelu(Modulex):
+class PRelu(Layer):
     def __init__(self,num_parameters=1,init=0.25):
         super(PRelu, self).__init__()
         self.num_parameters = num_parameters
-        self.weight = Parameter(torch.Tensor(num_parameters).fill_(init))
-        self._is_built = True
-    def forward(self, x):
-        return p_relu(x)
+        self.weight = Parameter(torch.Tensor(num_parameters))
+        nn.init.constant_(self.weight,0.02)
+        
+    def forward(self, *x):
+        x = enforce_singleton(x)
+        return p_relu(x,self.weight)
 
 '''p_relu activation function 
 '''
@@ -108,7 +116,7 @@ p_relu=torch.prelu
 
 '''Softmax activation function layer
 '''
-class Sigmoid(Modulex):
+class Sigmoid(Layer):
     """Softmax activation function.
        # Arguments
            x: Input tensor.
@@ -122,8 +130,9 @@ class Sigmoid(Modulex):
        """
     def __init__(self):
         super(Sigmoid, self).__init__()
-        self._is_built = True
-    def forward(self, x):
+        
+    def forward(self, *x):
+        x = enforce_singleton(x)
         return sigmoid(x)
 
 '''softmax activation function 
@@ -132,11 +141,12 @@ class Sigmoid(Modulex):
 def sigmoid(x):
     return  torch.sigmoid(x)
 
-class Tanh(Modulex):
+class Tanh(Layer):
     def __init__(self):
         super(Tanh, self).__init__()
-        self._is_built = True
-    def forward(self, x):
+        
+    def forward(self, *x):
+        x = enforce_singleton(x)
         return tanh(x)
 
 '''tanh activation function 
@@ -145,11 +155,12 @@ def tanh(x):
     return  torch.tanh(x)
 
 
-class Swish(Modulex):
+class Swish(Layer):
     def __init__(self):
         super(Swish, self).__init__()
-        self._is_built = True
-    def forward(self, x):
+        
+    def forward(self, *x):
+        x = enforce_singleton(x)
         return swish(x)
 
 
@@ -158,46 +169,50 @@ class Swish(Modulex):
 def swish(x):
     return x * sigmoid(x)
 
-class HardSigmoid(Modulex):
+class HardSigmoid(Layer):
     def __init__(self, inplace=False):
         super(HardSigmoid, self).__init__()
-        self._is_built = True
+        
         self.inplace = inplace
-    def forward(self, x):
+    def forward(self, *x):
+        x = enforce_singleton(x)
         return hard_sigmoid(x, inplace=self.inplace)
 
 def hard_sigmoid(x, inplace=False):
     return F.relu6(x + 3, inplace) / 6
 
-class HardSwish(Modulex):
+class HardSwish(Layer):
     def __init__(self, inplace=False):
         super(HardSwish, self).__init__()
-        self._is_built = True
+        
         self.inplace = inplace
-    def forward(self, x):
+    def forward(self, *x):
+        x = enforce_singleton(x)
         return hard_swish(x, inplace=self.inplace)
 
 def hard_swish(x, inplace=False):
     return x * hard_sigmoid(x, inplace)
 
 
-class HardTanh(Modulex):
+class HardTanh(Layer):
     def __init__(self,):
         super(HardTanh, self).__init__()
-        self._is_built = True
-    def forward(self, x):
+        
+    def forward(self, *x):
+        x = enforce_singleton(x)
         return hard_tanh(x)
 
 def hard_tanh(x):
     return  torch.clamp(x,-1,1)
 
 
-class Selu(Modulex):
+class Selu(Layer):
     def __init__(self, inplace=False):
         super(Selu, self).__init__()
-        self._is_built = True
+        
         self.inplace = inplace
-    def forward(self, x):
+    def forward(self, *x):
+        x = enforce_singleton(x)
         return selu(x)
 
 
@@ -207,11 +222,12 @@ def selu(x):
     return torch.selu(x)
 
 
-class Elu(Modulex):
+class Elu(Layer):
     def __init__(self):
         super(Elu, self).__init__()
-        self._is_built = True
-    def forward(self, x):
+        
+    def forward(self, *x):
+        x = enforce_singleton(x)
         return elu(x)
 
 
@@ -220,22 +236,24 @@ def elu(x):
 
 
 
-class LecunTanh(Modulex):
+class LecunTanh(Layer):
     def __init__(self):
         super(LecunTanh, self).__init__()
-        self._is_built = True
-    def forward(self, x):
+        
+    def forward(self, *x):
+        x = enforce_singleton(x)
         return hard_swish(x)
 
 def lecun_tanh(x):
     return 1.7159 * torch.tanh(2 / 3 * x)
 
 
-class SoftSign(Modulex):
+class SoftSign(Layer):
     def __init__(self):
         super(SoftSign, self).__init__()
-        self._is_built = True
-    def forward(self, x):
+        
+    def forward(self, *x):
+        x = enforce_singleton(x)
         return soft_sign(x)
 
 def soft_sign(x):
@@ -243,11 +261,12 @@ def soft_sign(x):
 
 
 
-class SoftPlus(Modulex):
+class SoftPlus(Layer):
     def __init__(self):
         super(SoftPlus, self).__init__()
-        self._is_built = True
-    def forward(self, x):
+        
+    def forward(self, *x):
+        x = enforce_singleton(x)
         return soft_plus(x)
 
 def soft_plus(x):
@@ -255,21 +274,23 @@ def soft_plus(x):
 
 
 
-class Logit(Modulex):
+class Logit(Layer):
     def __init__(self,):
         super(Logit, self).__init__()
-    def forward(self, x):
+    def forward(self, *x):
+        x = enforce_singleton(x)
         return logit(x)
 
 def logit(x):
     return  (x / (1 - x)).log()
 
 
-class LogLog(Modulex):
+class LogLog(Layer):
     def __init__(self,):
         super(LogLog, self).__init__()
-        self._is_built = True
-    def forward(self, x):
+        
+    def forward(self, *x):
+        x = enforce_singleton(x)
         return log_log(x)
 
 def log_log(x):
@@ -278,15 +299,16 @@ def log_log(x):
 
 
 
-class Mish(Modulex):
+class Mish(Layer):
     '''
         #Mish - "Mish: A Self Regularized Non-Monotonic Neural Activation Function"
         #https://arxiv.org/abs/1908.08681v1
     '''
     def __init__(self):
         super().__init__()
-        self._is_built = True
-    def forward(self, x):
+        
+    def forward(self, *x):
+        x = enforce_singleton(x)
         return mish(x)
 
 '''mish activation function 
@@ -295,10 +317,11 @@ def mish(x):
     return x *( torch.tanh(F.softplus(x)))
 
 
-class Softmax(Modulex):
+class Softmax(Layer):
     def __init__(self):
         super(Softmax, self).__init__()
-    def forward(self, x):
+    def forward(self, *x):
+        x = enforce_singleton(x)
         return softmax(x)
 
 def softmax(x):
@@ -307,25 +330,28 @@ def softmax(x):
 
 
 
-class BertGELU(Modulex):
+class BertGelu(Layer):
     r"""Bert uses GELU as the activation function for the position-wise network.
     """
     def __init__(self):
-        super(BertGELU, self).__init__()
-        self._is_built = True
-    def forward(self, x):
+        super(BertGelu, self).__init__()
+        
+    def forward(self, *x):
+        x = enforce_singleton(x)
         return bert_gelu(x)
 
 def bert_gelu(x):
     return x * 0.5 * (1.0 + torch.erf(x / math.sqrt(2.0)))
 
 
-class GPTGELU(Modulex):
+class GptGelu(Layer):
     r"""For information: OpenAI GPT's GELU is slightly different (and gives
     slightly different results).
     """
-    def forward(self, x):
-        self._is_built = True
+    def __init__(self):
+        super(GptGelu, self).__init__()
+    def forward(self, *x):
+        x = enforce_singleton(x)
         return gpt_gelu(x)
 
 def gpt_gelu(x):
@@ -335,14 +361,31 @@ def gpt_gelu(x):
 def get_activation(fn_name):
     if fn_name is None:
         return None
-    fn_modules = ['trident.layers.pytorch_activations']
+    fn_modules = ['trident.layers.pytorch_activations','torch.nn.functional']
     try:
-        if fn_name in __all__:
-            activation_fn = get_function(fn_name, fn_modules)
-            return activation_fn
+        if isinstance(fn_name,str):
+            if fn_name.lower()==fn_name:
+                 activation_fn = get_function(fn_name, ['trident.layers.pytorch_activations'] if fn_name in __all__ else  fn_modules)
+                 return activation_fn
+            else:
+                try:
+                    activation_fn = get_function(camel2snake(fn_name), fn_modules)
+                    return activation_fn()
+                except Exception:
+                    activation_fn = get_class(fn_name, ['trident.layers.pytorch_activations']if fn_name in __all__ else fn_modules)
+                    return activation_fn
+        elif getattr(fn_name, '__module__', None) == 'trident.layers.pytorch_activations':
+            if inspect.isfunction(fn_name):
+                return fn_name
+            elif isinstance(fn_name, Layer):
+                return fn_name()
         else:
-            activation_fn = get_function(camel2snake(fn_name), fn_modules)
-            return activation_fn
+            if callable(fn_name) :
+                result=inspect.getfullargspec(fn_name)
+                if 1<=len(result.args)<=2:
+                    return fn_name if inspect.isfunction(fn_name) else fn_name()
+                else:
+                    raise ValueError('Unknown activation function/ class')
     except Exception:
         return None
 
