@@ -11,7 +11,7 @@ import numpy as np
 from ..backend.common import get_session,addindent,get_time_suffix,get_class,get_function,camel2snake
 from ..backend.pytorch_ops import argmax
 
-__all__ = ['accuracy','psnr','mean_absolute_error','mean_squared_error','mean_squared_logarithmic_error','mae','mse','rmse','msle','get_metrics']
+__all__ = ['accuracy','pixel_accuracy','iou','psnr','mean_absolute_error','mean_squared_error','mean_squared_logarithmic_error','mae','mse','rmse','msle','get_metric']
 
 # def accuracy(input, target,axis=1):
 #     input_tensor=input.clone().detach()
@@ -30,14 +30,18 @@ __all__ = ['accuracy','psnr','mean_absolute_error','mean_squared_error','mean_sq
 
 
 
-def accuracy(output, target, topk=1,axis=1):
+def accuracy(output, target, topk=1,axis=1,exclude_mask=False):
     """Computes the precision@k for the specified values of k
     prec1, prec5 = accuracy(output.data, target, topk=(1, 5))
     """
     input_tensor=output.clone().detach()
     target_tensor=target.clone().detach()
+
     if input_tensor.dtype!=torch.int64 and topk==1:
-        input_tensor=argmax(input_tensor,axis).squeeze()
+        if len(input_tensor.size())==1: #binary
+            input_tensor=input_tensor.gt(0.5).float()
+        else:
+            input_tensor=argmax(input_tensor,axis).squeeze()
     if target_tensor.dtype!=torch.int64:
         target_tensor=argmax(target_tensor,axis).squeeze()
     if input_tensor.shape!=target_tensor.shape and topk==1:
@@ -118,15 +122,42 @@ def mean_squared_logarithmic_error(output, target):
 msle=mean_squared_logarithmic_error
 
 
-def get_metrics(metrics_name):
-    if metrics_name is None:
+
+def pixel_accuracy(output, target):
+    input_tensor = output.clone().detach()
+    target_tensor = target.clone().detach()
+    if input_tensor.dtype!=torch.int64 :
+        input_tensor=argmax(input_tensor,axis=1).squeeze()
+    pixel_labeled = (target_tensor > 0).sum().float()
+    pixel_correct = ((input_tensor == target_tensor)*(target_tensor > 0)).sum().float()
+    return pixel_correct/max(pixel_labeled,1)
+
+def iou(output, target):
+    input_tensor = output.clone().detach()
+    target_tensor = target.clone().detach()
+    if input_tensor.dtype != torch.int64:
+        input_tensor = argmax(input_tensor, axis=1).squeeze()
+
+    intersection =( (input_tensor > 0) * (input_tensor == target_tensor)).sum().float()
+    union=((input_tensor+target_tensor)>0).sum().float()
+
+    return intersection/max(union,1)
+
+
+
+def get_metric(metric_name):
+    if metric_name is None:
         return None
-    metrics_modules = ['trident.optims.pytorch_metrics']
-    try:
-        metrics_fn = get_function(camel2snake(metrics_name), metrics_modules)
-    except Exception :
-        metrics_fn = get_function(metrics_name, metrics_modules)
-    return metrics_fn
+
+    metric_modules = ['trident.optims.pytorch_metrics']
+    if metric_name in __all__:
+        metric_fn = get_function(metric_name, metric_modules)
+    else:
+        try:
+            metric_fn = get_function(camel2snake(metric_name), metric_modules)
+        except Exception :
+            metric_fn = get_function(metric_name, metric_modules)
+    return metric_fn
 
 
 
