@@ -24,6 +24,7 @@ __all__ = ['Adam','SGD','Adadelta','Adagrad','RMSprop','RAdam','PlainRAdam','Ada
 
 class Adam(optim.Adam,OptimizerMixin):
     pass
+
 class SGD(optim.SGD,OptimizerMixin):
     pass
 
@@ -127,7 +128,6 @@ class RAdam(Optimizer,OptimizerMixin):
         return loss
 
 class PlainRAdam(Optimizer,OptimizerMixin):
-
     def __init__(self, params, lr=1e-3, betas=(0.9, 0.999), eps=1e-8, weight_decay=0, degenerated_to_sgd=True):
         if not 0.0 <= lr:
             raise ValueError("Invalid learning rate: {}".format(lr))
@@ -475,9 +475,14 @@ def get_optimizer(optimizer_name):
     if optimizer_name is None:
         return None
     optimizer_modules = ['trident.optims.pytorch_optimizers','torch.optim']
-    try:
-        optimizer_class = get_class(snake2camel(optimizer_name), optimizer_modules)
-    except Exception :
+    if optimizer_name in __all__:
         optimizer_class = get_class(optimizer_name, optimizer_modules)
-    return optimizer_class
+        return optimizer_class
+    else:
+        try:
+            optimizer_class = get_class(snake2camel(optimizer_name), optimizer_modules)
+            return optimizer_class
+        except Exception :
+            optimizer_class = get_class(optimizer_name, optimizer_modules)
+        return optimizer_class
 
